#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from persistent import Persistent

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphFactory
from pyams_content.component.paragraph.interfaces.keynumber import IKeyNumber, IKeyNumberContainer, \
    IKeyNumberContainerTarget, KEYNUMBER_CONTAINER_KEY, IKeyNumberParagraph, KEYNUMBER_PARAGRAPH_TYPE, \
    KEYNUMBER_PARAGRAPH_RENDERERS
from pyams_content.features.checker.interfaces import IContentChecker, MISSING_VALUE, MISSING_LANG_VALUE
from pyams_form.interfaces.form import IFormContextPermissionChecker
from pyams_i18n.interfaces import II18n, II18nManager, INegotiator
from zope.annotation.interfaces import IAnnotations
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent
from zope.location.interfaces import ISublocations
from zope.traversing.interfaces import ITraversable

# import packages
from pyams_catalog.utils import index_object
from pyams_content.component.paragraph import BaseParagraph, BaseParagraphFactory, BaseParagraphContentChecker
from pyams_content.features.checker import BaseContentChecker
from pyams_content.features.renderer import RenderedContentMixin, IContentRenderer
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.registry import get_current_registry, get_utility, utility_config
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.container.ordered import OrderedContainer
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent, ObjectModifiedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

from pyams_content import _


#
# Key number class and adapters
#

@implementer(IKeyNumber)
class KeyNumber(Persistent, Contained):
    """Key number persistent class"""

    visible = FieldProperty(IKeyNumber['visible'])
    number = FieldProperty(IKeyNumber['number'])
    label = FieldProperty(IKeyNumber['label'])
    text = FieldProperty(IKeyNumber['text'])


@adapter_config(context=IKeyNumber, provides=IFormContextPermissionChecker)
class KeyNumberPermissionChecker(ContextAdapter):
    """Key number permission checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, IKeyNumberContainerTarget)
        return IFormContextPermissionChecker(content).edit_permission


@subscriber(IObjectAddedEvent, context_selector=IKeyNumber)
def handle_added_keynumber(event):
    """Handle added key number"""
    content = get_parent(event.object, IKeyNumberContainerTarget)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectModifiedEvent, context_selector=IKeyNumber)
def handle_modified_keynumber(event):
    """Handle modified key number"""
    content = get_parent(event.object, IKeyNumberContainerTarget)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectRemovedEvent, context_selector=IKeyNumber)
def handle_removed_keynumber(event):
    """Handle removed key number"""
    content = get_parent(event.object, IKeyNumberContainerTarget)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@adapter_config(context=IKeyNumber, provides=IContentChecker)
class KeyNumberContentChecker(BaseContentChecker):
    """Key number content checker"""

    @property
    def label(self):
        request = check_request()
        return II18n(self.context).query_attribute('title', request=request)

    def inner_check(self, request):
        output = []
        translate = request.localizer.translate
        manager = get_parent(self.context, II18nManager)
        if manager is not None:
            langs = manager.get_languages()
        else:
            negotiator = get_utility(INegotiator)
            langs = (negotiator.server_language, )
        i18n = II18n(self.context)
        for lang in langs:
            for attr in ('label', 'text'):
                value = i18n.get_attribute(attr, lang, request)
                if not value:
                    field_title = translate(IKeyNumber[attr].title)
                    if len(langs) == 1:
                        output.append(translate(MISSING_VALUE).format(field=field_title))
                    else:
                        output.append(translate(MISSING_LANG_VALUE).format(field=field_title, lang=lang))
        return output


#
# Key numbers container classes and adapters
#

@implementer(IKeyNumberContainer)
class KeyNumberContainer(OrderedContainer):
    """Key numbers container"""

    last_id = 1

    def append(self, value, notify=True):
        key = str(self.last_id)
        if not notify:
            # pre-locate key number item to avoid multiple notifications
            locate(value, self, key)
        self[key] = value
        self.last_id += 1
        if not notify:
            # make sure that key number item is correctly indexed
            index_object(value)

    def get_visible_items(self):
        return filter(lambda x: IKeyNumber(x).visible, self.values())


@adapter_config(context=IKeyNumberContainerTarget, provides=IKeyNumberContainer)
def keynumber_container_factory(target):
    """Key number container factory"""
    annotations = IAnnotations(target)
    container = annotations.get(KEYNUMBER_CONTAINER_KEY)
    if container is None:
        container = annotations[KEYNUMBER_CONTAINER_KEY] = KeyNumberContainer()
        get_current_registry().notify(ObjectCreatedEvent(container))
        locate(container, target, '++keynumbers++')
    return container


@adapter_config(name='keynumbers', context=IKeyNumberContainerTarget, provides=ITraversable)
class KeyNumberContainerNamespace(ContextAdapter):
    """Key numbers container ++keynumbers++ namespace"""

    def traverse(self, name, furtherpaath=None):
        return IKeyNumberContainer(self.context)


@adapter_config(name='keynumbers', context=IKeyNumberContainerTarget, provides=ISublocations)
class KeyNumberContainerSublocations(ContextAdapter):
    """Key numbers container sub-locations adapter"""

    def sublocations(self):
        return IKeyNumberContainer(self.context).values()


@adapter_config(name='keynumbers', context=IKeyNumberContainerTarget, provides=IContentChecker)
class KeyNumberContainerContentChecker(BaseContentChecker):
    """Key numbers container content checker"""

    label = _("Key numbers")
    sep = '\n'
    weight = 200

    def inner_check(self, request):
        output = []
        registry = request.registry
        for keynumber in IKeyNumberContainer(self.context).values():
            if not keynumber.visible:
                continue
            for name, checker in sorted(registry.getAdapters((keynumber, ), IContentChecker),
                                        key=lambda x: x[1].weight):
                output.append('- {0} ({1}):'.format(keynumber.number,
                                                    II18n(keynumber).query_attribute('label', request=request) or '--'))
                output.append(checker.get_check_output(request))
        return output


@implementer(IKeyNumberParagraph)
class KeyNumberParagraph(RenderedContentMixin, BaseParagraph):
    """Key numbers paragraph"""

    icon_class = 'fa-list-ol'
    icon_hint = _("Key numbers")

    renderer = FieldProperty(IKeyNumberParagraph['renderer'])


@utility_config(name=KEYNUMBER_PARAGRAPH_TYPE, provides=IParagraphFactory)
class KeyNumberParagraphFactory(BaseParagraphFactory):
    """Key numbers paragraph factory"""

    name = _("Key numbers paragraph")
    content_type = KeyNumberParagraph


@adapter_config(context=IKeyNumberParagraph, provides=IContentChecker)
class KeyNumberParagraphContentChecker(BaseParagraphContentChecker):
    """Key numbers paragraph content checker"""

    @property
    def label(self):
        request = check_request()
        translate = request.localizer.translate
        return II18n(self.context).query_attribute('title', request) or \
            '({0})'.format(translate(self.context.icon_hint).lower())

    def inner_check(self, request):
        output = []
        translate = request.localizer.translate
        manager = get_parent(self.context, II18nManager)
        if manager is not None:
            langs = manager.get_languages()
        else:
            negotiator = get_utility(INegotiator)
            langs = (negotiator.server_language, )
        i18n = II18n(self.context)
        for lang in langs:
            value = i18n.get_attribute('title', lang, request)
            if not value:
                field_title = translate(IKeyNumberParagraph['title'].title)
                if len(langs) == 1:
                    output.append(translate(MISSING_VALUE).format(field=field_title))
                else:
                    output.append(translate(MISSING_LANG_VALUE).format(field=field_title, lang=lang))
        return output


@vocabulary_config(name=KEYNUMBER_PARAGRAPH_RENDERERS)
class KeyNumberParagraphRendererVocabulary(SimpleVocabulary):
    """Key numbers paragraph renderers vocabulary"""

    def __init__(self, context=None):
        request = check_request()
        translate = request.localizer.translate
        registry = request.registry
        if not IKeyNumberParagraph.providedBy(context):
            context = KeyNumberParagraph()
        terms = [SimpleTerm(name, title=translate(adapter.label))
                 for name, adapter in sorted(registry.getAdapters((context, request), IContentRenderer),
                                             key=lambda x: x[1].weight)]
        super(KeyNumberParagraphRendererVocabulary, self).__init__(terms)
