#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IBaseParagraph, IParagraphFactory, \
    IParagraphFactorySettings
from pyams_content.component.paragraph.zmi.interfaces import IParagraphContainerView
from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_form.interfaces.form import IFormHelp
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.container import ITableElementName
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer

# import packages
from pyams_content.component.paragraph.zmi.container import ParagraphContainerTable, ParagraphContainerBaseTable
from pyams_form.form import AJAXEditForm, AJAXAddForm
from pyams_form.help import FormHelp
from pyams_form.security import ProtectedFormObjectMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.event import get_json_switched_table_refresh_event
from pyams_skin.table import get_element_id
from pyams_skin.viewlet.menu import MenuItem, MenuDivider
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogEditForm
from pyramid.location import lineage
from pyramid.view import view_config
from z3c.form import field

from pyams_content import _


#
# Default paragraphs settings
#

@viewlet_config(name='default-paragraphs.menu', context=IParagraphFactorySettings, layer=IAdminLayer,
                manager=IPropertiesMenu, permission=MANAGE_TOOL_PERMISSION, weight=10)
class DefaultParagraphsSettingsMenu(MenuItem):
    """Default paragraphs settings menu"""

    label = _("Content block types...")
    icon_class = 'fa-paragraph'
    url = 'default-paragraphs.html'
    modal_target = True


@pagelet_config(name='default-paragraphs.html', context=IParagraphFactorySettings, layer=IPyAMSLayer,
                permission=MANAGE_TOOL_PERMISSION)
class DefaultParagraphsEditForm(AdminDialogEditForm):
    """Default paragraphs edit form"""

    prefix = 'default_paragraphs.'

    legend = _("Content block types")

    fields = field.Fields(IParagraphFactorySettings)
    ajax_handler = 'default-paragraphs.json'
    edit_permission = MANAGE_TOOL_PERMISSION


@view_config(name='default-paragraphs.json', context=IParagraphFactorySettings, request_type=IPyAMSLayer,
             permission=MANAGE_TOOL_PERMISSION, renderer='json', xhr=True)
class DefaultParagraphAJAXEditForm(AJAXEditForm, DefaultParagraphsEditForm):
    """Default paragraphs edit form, JSON renderer"""


@adapter_config(context=(IParagraphFactorySettings, IPyAMSLayer, DefaultParagraphsEditForm), provides=IFormHelp)
class DefaultParagraphsEditFormHelp(FormHelp):
    """Default paragraphs edit form help"""

    message = _("You can define which types of paragraphs are allowed in this container.\n\n"
                "Default paragraphs will be added automatically (in selected order) to any new created content.\n\n"
                "NOTICE: removing types from allowed types list will have no effect on already created contents!")
    message_format = 'rest'


#
# Base paragraph forms
#

def get_json_paragraph_refresh_event(context, request):
    """Get JSON response value for paragraph refresh event"""
    parent = get_parent(context, IParagraphContainerTarget)
    return {
        'event': 'myams.refresh',
        'options': {
            'handler': 'PyAMS_content.paragraphs.refreshParagraph',
            'object_id': get_element_id(ParagraphContainerBaseTable, context, parent),
            'title': II18n(context).query_attribute('title', request=request),
            'visible': context.visible
        }
    }


def get_json_paragraph_toolbar_refresh_event(context, request, table_factory=None, viewlet_factory=None):
    """Get JSON response value for paragraph toolbar refresh event"""
    if table_factory is None:
        from pyams_content.component.paragraph.zmi.container import ParagraphContainerTable as table_factory
    if viewlet_factory is None:
        from pyams_content.component.paragraph.zmi.container import ParagraphTitleToolbarViewletManager as viewlet_factory

    parent = get_parent(context, IParagraphContainerTarget)
    table = table_factory(context, request)
    viewlet = viewlet_factory(context, request, table)
    viewlet.update()
    return {
        'event': 'myams.refresh',
        'options': {
            'handler': 'PyAMS_content.paragraphs.updateToolbar',
            'object_id': get_element_id(ParagraphContainerBaseTable, context, parent),
            'toolbar_tag': viewlet.render()
        }
    }


def get_json_paragraph_markers_refresh_event(context, request, form, viewlet_factory, marker_type=None):
    """Get JSON response value for paragraph markers refresh event"""
    parent = get_parent(context, IParagraphContainerTarget)
    marker = viewlet_factory(context, request, form, None)
    if marker is not None:
        marker.update()
        return {
            'event': 'myams.refresh',
            'options': {
                'handler': 'PyAMS_content.paragraphs.updateMarkers',
                'object_id': get_element_id(ParagraphContainerBaseTable, context, parent),
                'marker_type': marker_type or marker.marker_type,
                'marker_tag': marker.render()
            }
        }


class BaseParagraphAddMenu(ProtectedFormObjectMixin, ToolbarMenuItem):
    """Base paragraph add menu"""

    paragraph_type = None
    modal_target = True

    def __new__(cls, context, request, view, manager):
        settings = get_parent(context, IParagraphFactorySettings)
        if (settings is not None) and (cls.paragraph_type not in (settings.allowed_paragraphs or ())):
            return None
        return ToolbarMenuItem.__new__(cls)


@viewlet_config(name='custom-paragraphs.divider', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=500)
class ParagraphsAddMenuDivider(ProtectedFormObjectMixin, MenuDivider):
    """Paragraphs add menu divider"""

    def __new__(cls, context, request, view, manager):
        settings = get_parent(context, IParagraphFactorySettings)
        if settings is None:
            return MenuDivider.__new__(cls)
        for factory_name in settings.allowed_paragraphs or ():
            factory = query_utility(IParagraphFactory, name=factory_name)
            if factory.secondary_menu:
                return MenuDivider.__new__(cls)
        return None


class BaseParagraphAJAXAddForm(AJAXAddForm):
    """Base paragraph AJAX add form"""

    def get_ajax_output(self, changes):
        if IWfSharedContent.providedBy(self.context):
            table_factory = ParagraphContainerTable
        else:
            table_factory = ParagraphContainerBaseTable
        return {
            'status': 'success',
            'message': self.request.localizer.translate(_("Paragraph was correctly added.")),
            'events': [get_json_switched_table_refresh_event(self.context, self.request, table_factory)]
        }


class BaseParagraphPropertiesEditForm(AdminDialogEditForm):
    """Base paragraph edit form"""

    prefix = 'paragraph.'

    @property
    def title(self):
        registry = self.request.registry
        for parent in lineage(self.context):
            adapter = registry.queryMultiAdapter((parent, self.request), ITableElementName)
            if adapter is not None:
                return adapter.name
        content = get_parent(self.context, IWfSharedContent)
        if content is not None:
            return II18n(content).query_attribute('title', request=self.request)
        return super(BaseParagraphPropertiesEditForm, self).title


class BaseParagraphAJAXEditForm(AJAXEditForm):
    """Base paragraph AJAX edit form"""

    def get_ajax_output(self, changes):
        output = super(BaseParagraphAJAXEditForm, self).get_ajax_output(changes)
        if 'title' in changes.get(IBaseParagraph, ()):
            output.setdefault('events', []).append(
                get_json_paragraph_refresh_event(self.context, self.request))
        return output
