#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IBaseParagraph, IParagraphRenderer
from pyams_content.component.paragraph.zmi.interfaces import IParagraphContainerView, IParagraphInnerEditor
from pyams_content.component.video.interfaces import IExternalVideoProvider, IExternalVideoSettings, \
    IExternalVideoParagraph, IExternalVideoRenderer, EXTERNAL_VIDEO_PARAGRAPH_TYPE
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IWidgetsSuffixViewletsManager, IInnerForm, IEditFormButtons
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces.data import IObjectData
from z3c.form.interfaces import DISPLAY_MODE, INPUT_MODE, IDataExtractedEvent

# import packages
from pyams_content.component.paragraph.zmi import BaseParagraphAddMenu, BaseParagraphAJAXAddForm, \
    BaseParagraphPropertiesEditForm, BaseParagraphAJAXEditForm, get_json_paragraph_refresh_event
from pyams_content.component.video.paragraph import ExternalVideoParagraph
from pyams_form.group import NamedWidgetsGroup
from pyams_pagelet.pagelet import pagelet_config
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import get_utility, get_current_registry
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config, Viewlet, BaseContentProvider
from pyams_zmi.form import AdminDialogAddForm, InnerAdminAddForm, InnerAdminEditForm
from pyramid.events import subscriber
from pyramid.exceptions import NotFound
from pyramid.response import Response
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer, alsoProvides, Interface, Invalid
from zope.schema import getFieldNamesInOrder

from pyams_content import _


@viewlet_config(name='add-external-video.menu', context=IParagraphContainerTarget, view=IParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=75)
class ExternalVideoParagraphAddMenu(BaseParagraphAddMenu):
    """External video paragraph add menu"""

    label = _("External video...")
    label_css_class = 'fa fa-fw fa-youtube-play'
    url = 'add-external-video.html'
    paragraph_type = EXTERNAL_VIDEO_PARAGRAPH_TYPE


@pagelet_config(name='add-external-video.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class ExternalVideoParagraphAddForm(AdminDialogAddForm):
    """External video paragraph add form"""

    legend = _("Add new external video...")
    dialog_class = 'modal-large'
    icon_css_class = 'fa fa-fw fa-youtube-play'

    fields = field.Fields(IExternalVideoParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-external-video.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ExternalVideoParagraphAddForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'
        if 'body' in self.widgets:
            self.widgets['body'].label = ''
            self.add_group(NamedWidgetsGroup(self, 'body_group', self.widgets, ('body',),
                                             bordered=False,
                                             legend=_("HTML content"),
                                             css_class='inner switcher padding-right-10 no-y-padding pull-left',
                                             switch=True,
                                             hide_if_empty=True))
            self.add_group(NamedWidgetsGroup(self, 'data_group', self.widgets,
                                             ('description', 'author', 'provider_name'),
                                             bordered=False))
        if 'provider_name' in self.widgets:
            widget = self.widgets['provider_name']
            widget.object_data = {
                'ams-change-handler': 'MyAMS.helpers.select2ChangeHelper',
                'ams-change-stop-propagation': 'true',
                'ams-select2-helper-type': 'html',
                'ams-select2-helper-url': absolute_url(self.context, self.request,
                                                       'get-video-provider-settings-add-form.html'),
                'ams-select2-helper-argument': 'provider_name',
                'ams-select2-helper-target': '#video-settings-helper'
            }
            alsoProvides(widget, IObjectData)

    def create(self, data):
        return ExternalVideoParagraph()

    def update_content(self, content, data):
        changes = super(ExternalVideoParagraphAddForm, self).update_content(content, data)
        settings = IExternalVideoSettings(content, None)
        if settings is not None:
            provider = content.get_provider()
            form = InnerAdminEditForm(settings, self.request)
            form.prefix = self.prefix
            form.edit_permission = MANAGE_CONTENT_PERMISSION
            form.fields = field.Fields(provider.settings_interface)
            form.update()
            settings_data, errors = form.extractData()
            if not errors:
                changes.update(form.update_content(settings, settings_data))
        return changes

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-external-video.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExternalVideoParagraphAJAXAddForm(BaseParagraphAJAXAddForm, ExternalVideoParagraphAddForm):
    """External video paragraph add form, JSON renderer"""


@subscriber(IDataExtractedEvent, form_selector=ExternalVideoParagraphAddForm)
def handle_video_paragraph_add_form_data_extraction(event):
    """Handle provider name data extraction"""
    data = event.data
    if not data.get('provider_name'):
        event.form.widgets.errors += (Invalid(_("Video provider is required")), )


@viewlet_config(name='external-video-settings', context=Interface, layer=IPyAMSLayer,
                view=ExternalVideoParagraphAddForm, manager=IWidgetsSuffixViewletsManager)
@template_config(template='templates/video-settings.pt', layer=IPyAMSLayer)
class VideoSettingsWidgetsSuffix(Viewlet):
    """External video settings edit form widgets suffix"""


@view_config(name='get-video-provider-settings-add-form.html', context=IParagraphContainerTarget,
             request_type=IPyAMSLayer, permission=MANAGE_CONTENT_PERMISSION, xhr=True)
def external_video_provider_settings_add_form(request):
    """External video provider settings form"""
    provider_name = request.params.get('provider_name')
    if provider_name is None:
        raise NotFound("No provided provider_name argument")
    elif (not provider_name) or (provider_name == '--NOVALUE--'):
        return Response('')
    else:
        provider = get_utility(IExternalVideoProvider, name=provider_name)
        form = InnerAdminAddForm(request.context, request)
        form.prefix = ExternalVideoParagraphAddForm.prefix
        form.legend = request.localizer.translate(_("Video provider settings"))
        form.label_css_class = 'control-label col-md-4'
        form.input_css_class = 'col-md-8'
        form.fields = field.Fields(provider.settings_interface)
        form.update()
        if 'integration_code' in form.widgets:  # custom video provider
            form.widgets['integration_code'].widget_css_class = 'textarea'
            form.add_group(NamedWidgetsGroup(form, 'integration_code', form.widgets, ('integration_code', ),
                                             bordered=False))
        elif 'video_id' in form.widgets:
            form.add_group(NamedWidgetsGroup(form, 'video_id', form.widgets, ('video_id', ), bordered=False))
        field_names = getFieldNamesInOrder(provider.settings_interface)
        if len(field_names) > 1:
            form.add_group(NamedWidgetsGroup(form, 'provider_group', form.widgets,
                                             field_names[1:],
                                             bordered=False,
                                             legend=_("Other settings"),
                                             css_class="inner switcher padding-right-10 no-y-padding",
                                             switch=True,
                                             hide_if_empty=True))
        return Response(form.render())


@pagelet_config(name='properties.html', context=IExternalVideoParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class ExternalVideoParagraphPropertiesEditForm(BaseParagraphPropertiesEditForm):
    """External video paragraph properties edit form"""

    prefix = 'externalvideo_properties.'

    legend = _("Edit video properties")
    icon_css_class = 'fa fa-fw fa-youtube-play'

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    @property
    def fields(self):
        fields = field.Fields(IExternalVideoParagraph).omit('__parent__', '__name__', 'visible')
        provider = self.context.get_provider()
        if provider is not None:
            fields += field.Fields(provider.settings_interface)
        return fields

    def updateWidgets(self, prefix=None):
        super(ExternalVideoParagraphPropertiesEditForm, self).updateWidgets(prefix)
        if 'description' in self.widgets:
            self.widgets['description'].widget_css_class = 'textarea'
        if 'body' in self.widgets:
            self.widgets['body'].label = ''
            self.add_group(NamedWidgetsGroup(self, 'body_group', self.widgets, ('body',),
                                             bordered=False,
                                             fieldset_class='margin-top-10 padding-y-5',
                                             legend=_("HTML content"),
                                             css_class='inner switcher padding-right-10 no-y-padding pull-left',
                                             switch=True,
                                             hide_if_empty=True))
            self.add_group(NamedWidgetsGroup(self, 'data_group', self.widgets,
                                             ('description', 'author', 'provider_name'),
                                             bordered=False))
        if 'provider_name' in self.widgets:
            self.widgets['provider_name'].mode = DISPLAY_MODE
            provider = self.context.get_provider()
            if provider is not None:
                if 'integration_code' in self.widgets:  # custom video provider
                    self.widgets['integration_code'].widget_css_class = 'textarea'
                    self.add_group(NamedWidgetsGroup(self, 'integration_code', self.widgets, ('integration_code',),
                                                     bordered=False))
                elif 'video_id' in self.widgets:
                    self.add_group(NamedWidgetsGroup(self, 'video_id', self.widgets, ('video_id',), bordered=False))
                field_names = getFieldNamesInOrder(provider.settings_interface)
                if len(field_names) > 1:
                    self.add_group(NamedWidgetsGroup(self, 'provider_group', self.widgets,
                                                     field_names[1:],
                                                     bordered=False,
                                                     legend=_("Video provider settings"),
                                                     fieldset_class='margin-top-10 padding-y-5',
                                                     css_class='inner switcher padding-right-10 no-y-padding pull-left',
                                                     switch=True,
                                                     hide_if_empty=True))


@view_config(name='properties.json', context=IExternalVideoParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExternalVideoParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, ExternalVideoParagraphPropertiesEditForm):
    """External video paragraph properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(ExternalVideoParagraphPropertiesAJAXEditForm, self).get_ajax_output(changes)
        if 'title' in changes.get(IBaseParagraph, ()):
            output.setdefault('events', []).append(
                get_json_paragraph_refresh_event(self.context, self.request))
        return output


@adapter_config(context=(IExternalVideoParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm)
class ExternalVideoParagraphInnerEditForm(ExternalVideoParagraphPropertiesEditForm):
    """External video paragraph properties inner deit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IExternalVideoParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class ExternalVideoParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, ExternalVideoParagraphInnerEditForm):
    """External video paragraph inner edit form, JSON renderer"""


#
# Video paragraph renderer
#

@adapter_config(context=(IExternalVideoParagraph, IPyAMSLayer), provides=IParagraphRenderer)
@template_config(template='templates/video-render.pt', layer=IPyAMSLayer)
class ExternalVideoParagraphRenderer(BaseContentProvider):
    """External video paragraph renderer"""

    video_renderer = None

    def __init__(self, context, request):
        super(ExternalVideoParagraphRenderer, self).__init__(context, request)
        provider = context.get_provider()
        if provider is not None:
            registry = get_current_registry()
            self.video_renderer = registry.queryMultiAdapter((context.settings, request), IExternalVideoRenderer)

    def update(self):
        i18n = II18n(self.context)
        if self.language:
            for attr in ('title', 'body', 'description'):
                setattr(self, attr, i18n.get_attribute(attr, self.language, request=self.request))
        else:
            for attr in ('title', 'body', 'description'):
                setattr(self, attr, i18n.query_attribute(attr, request=self.request))
        renderer = self.video_renderer
        if renderer is not None:
            renderer.update()

    def render_video(self):
        renderer = self.video_renderer
        if not renderer:
            return ''
        return renderer.render()
