#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.features.renderer.interfaces import IRenderedContent, IContentRenderer, IRendererSettings
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_i18n.interfaces import II18n
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_form.form import AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_template.template import get_view_template
from pyams_utils.adapter import ContextRequestAdapter, adapter_config
from pyams_viewlet.viewlet import BaseContentProvider
from pyams_zmi.form import AdminDialogEditForm
from pyramid.decorator import reify
from pyramid.view import view_config
from z3c.form import field
from zope.interface import implementer, Interface

from pyams_content import _


#
# Base rendered content renderer
#

class BaseRenderedContentRenderer(BaseContentProvider):
    """Base content renderer"""

    language = None

    def __init__(self, context, request):
        super(BaseRenderedContentRenderer, self).__init__(context, request)
        self.renderer = self.context.get_renderer()

    def update(self):
        renderer = self.renderer
        if renderer is not None:
            renderer.language = self.language
            renderer.update()

    def render(self):
        if self.renderer is None:
            return ''
        return self.renderer.render()


#
# Base content renderer
#

@implementer(IContentRenderer)
class BaseContentRenderer(ContextRequestAdapter):
    """Base content renderer"""

    label = None
    weight = 0
    settings_interface = None

    language = None
    context_attrs = ()
    i18n_context_attrs = ()

    @reify
    def settings(self):
        if self.settings_interface is None:
            return None
        return IRendererSettings(self.context)

    def update(self):
        for attr in self.context_attrs:
            setattr(self, attr, getattr(self.context, attr, None))
        if self.i18n_context_attrs:
            i18n = II18n(self.context, None)
            if i18n is not None:
                for attr in self.i18n_context_attrs:
                    setattr(self, attr, i18n.get_attribute(attr, lang=self.language, request=self.request))

    render = get_view_template()


@pagelet_config(name='renderer-properties.html', context=IRenderedContent, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class RendererPropertiesEditForm(AdminDialogEditForm):
    """Renderer properties edit form"""

    prefix = 'renderer_properties.'

    legend = _("Edit renderer properties")
    icon_css_class = 'fa fa-fw fa-pencil-square-o'

    @property
    def fields(self):
        renderer = IContentRenderer(self.context)
        return field.Fields(renderer.settings_interface or Interface)

    ajax_handler = 'renderer-properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def getContent(self):
        return IRendererSettings(self.context)


@view_config(name='renderer-properties.json', context=IRenderedContent, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class RendererPropertiesAJAXEditForm(AJAXEditForm, RendererPropertiesEditForm):
    """Renderer properties edit form, JSON renderer"""


#
# Default common renderers
#

@adapter_config(name='hidden', context=(IRenderedContent, IPyAMSLayer), provides=IContentRenderer)
class HiddenContentRenderer(BaseContentRenderer):
    """Hidden content renderer"""

    label = _("Hidden content")
    weight = -999

    def render(self):
        return ''
